import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:pichosting_admin_panel/models/enums/connection_status.dart';
import 'package:pichosting_admin_panel/models/picture.dart';
import 'package:pichosting_admin_panel/screens/edit.dart';
import 'package:pichosting_admin_panel/screens/widgets/search_app_bar.dart';
import 'package:pichosting_admin_panel/state/search/search_cubit.dart';
import 'package:pichosting_admin_panel/state/search/search_state.dart';
import 'package:responsive_grid_list/responsive_grid_list.dart';
import 'package:transparent_image/transparent_image.dart';

class HomePage extends StatefulWidget {
  const HomePage({Key? key}) : super(key: key);

  @override
  State<HomePage> createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  late final TextEditingController _controller;

  @override
  void initState() {
    _controller = TextEditingController();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<SearchCubit, SearchState>(
      builder: (context, state) {
        return Scaffold(
          appBar: SearchAppBar(
            textController: _controller,
            onUpload:  () {
              context.read<SearchCubit>().resetState();
            },
            height: 150,
            onTextChanged: (val) =>
                context.read<SearchCubit>().search(page: 1, query: val),
          ),
          backgroundColor: const Color(0xFF292929),
          body: SingleChildScrollView(
            child: SafeArea(
              child: Padding(
                padding: const EdgeInsets.only(
                    top: 20, bottom: 20, left: 100, right: 100),
                child: _buildSearchResults(
                  results: state.searchResults,
                  query: state.currentQuery,
                  currentPageKey: state.page,
                  status: state.status,
                ),
              ),
            ),
          ),
        );
      },
    );
  }

  Widget _buildSearchResults({
    required List<Picture> results,
    required int currentPageKey,
    required String query,
    required ConnectionStatus status,
  }) {
    return Padding(
      padding: const EdgeInsets.only(top: 20),
      child: ResponsiveGridListBuilder(
        gridItems: results
            .map((pic) => GestureDetector(
                  onTap: () {
                    context.read<SearchCubit>().resetState();
                    Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => EditScreen(
                        imageUrl: pic.file_url,
                        tags: pic.tags,
                        uuid: pic.uuid,
                      ),
                    ),
                  );},
                  child: FadeInImage.memoryNetwork(
                    image: pic.thumbnail_url,
                    placeholder: kTransparentImage,
                    width: 150,
                    height: 150,
                    fit: BoxFit.cover,
                  ),
                ))
            .toList(),
        builder: (BuildContext context, List<Widget> items) {
          return Column(
            children: [
              ...items,
              const SizedBox(
                height: 30,
              ),
              if (results.isNotEmpty)
                _buildUploadMoreButton(
                  context,
                  query,
                  currentPageKey,
                )
            ],
          );
        },
        minItemWidth: 150,
      ),
    );
  }

  Widget _buildUploadMoreButton(
      BuildContext context, String query, int currentPageKey) {
    return TextButton(
      onPressed: () {
        context
            .read<SearchCubit>()
            .search(page: currentPageKey + 1, query: query);
      },
      child: const Text(
        'Загрузить еще',
        style: TextStyle(color: Colors.white),
      ),
      style: TextButton.styleFrom(
          textStyle: const TextStyle(
            fontSize: 13,
            color: Colors.white,
          ),
          minimumSize: Size(MediaQuery.of(context).size.width / 2, 40),
          maximumSize: Size(MediaQuery.of(context).size.width, 40),
          backgroundColor: Colors.black),
    );
  }
}
